==> bakery-0.3.0.tgz: include/bakery/App/App_WithDoc.h <==
/*
 * Copyright 2000 Murray Cumming
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef APP_WITHDOC_H
#define APP_WITHDOC_H

#include "App.h"
#include "../Document/Document.h"

namespace Bakery
{

//Features:
//- Derive from this class
//- 1 document per application instance. Uses Document-derived class polymorphically.
//- Override pure virtual method to create new blank document.
//- Appropriate Default handling of document open, save, save as.
//- Appropriate checking of document 'modified' status - asks user about unsaved changes.
//- Asks user about overwriting existing documents.
//- Override methods to add/change menus/toolbars/statusbar.
//  - Default is basic File, Edit, Help menus and toolbar icons.
//- Shows document name (or 'untitled') in window title.
//- Shows * in title bar for unsaved docs. Overridable to e.g. shade a Save icon.

//TODO:
//- Enforce a file extension.
//- Recent documents - when GConf is ready.
//- Bonobo integration - when Bonobo is ready. (An App subclass?)
//- Printing - GnomePrint?
//  - Print Setup
//  - Print Preview
//- Multiple document-types:
//  - File/New sub menu
//  - Some way to associate a view with a document type
//  Gnome::MDI App.

class App_WithDoc : public App
{
public: 
	App_WithDoc(const string& appname, const string& title); //Don't forget to call init() too.
	virtual ~App_WithDoc();
	
protected:
  virtual void init(); //overriden to create document.
  virtual void init_menus_file(); //overridden to add open/save/save as.
  virtual void init_toolbars(); //overridden to add open/save
	virtual void init_create_document(); //override this to new() the specific document type.

  virtual Document* get_document(); //static_cast<> or dynamic_cast<> this pointer to the correct type.

  //Signal handlers:

  //Menu items:
  virtual void on_menu_File_Open();
  virtual void on_menu_File_SaveAs();
  virtual void on_menu_File_Save();
  virtual void on_menu_File_Close();

  //FileSelections (Open, Save):
  virtual void on_FileSelection_Open_OK();
  virtual void on_FileSelection_Open_Cancel();
  virtual void on_FileSelection_Save_OK();
  virtual void on_FileSelection_Save_Cancel();

  //Document:
  virtual void on_Document_modified(); //Update visual status.
  virtual void on_Document_load(); //override this to show document contents.

  virtual void offer_to_save_changes();
  virtual void update_window_title();

  //Document:
  Document* m_pDocument; //An instance of a derived type.
  bool m_bCloseAfterSave;

  //Member widets:
  Gtk::FileSelection m_FileSelection_Open, m_FileSelection_Save;
};

} //namespace

#endif

==> bakery-0.6.0.tgz: include/bakery/App/App_WithDoc.h <==
/*
 * Copyright 2000 Murray Cumming
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef APP_WITHDOC_H
#define APP_WITHDOC_H

#include "App.h"
#include "../Document/Document.h"
#include "../FileSelection/FileSelection.h"

namespace Bakery
{

//Features:
//- Derive from this class
//- 1 document per application instance. Uses Document-derived class polymorphically.
//- Override pure virtual method to create new blank document.
//- Appropriate Default handling of document open, save, save as.
//- Appropriate checking of document 'modified' status - asks user about unsaved changes.
//- Asks user about overwriting existing documents.
//- Override methods to add/change menus/toolbars/statusbar.
//  - Default is basic File, Edit, Help menus and toolbar icons.
//- Shows document name (or 'untitled') in window title.
//- Shows * in title bar for unsaved docs. Overridable to e.g. shade a Save icon.

//TODO:
//- Enforce a file extension.
//- Recent documents - when GConf is ready.
//- Bonobo integration - when Bonobo is ready. (An App subclass?)
//- Printing - GnomePrint?
//  - Print Setup
//  - Print Preview
//- Multiple document-types:
//  - File/New sub menu
//  - Some way to associate a view with a document type: class factory.
//  Gnome::MDI App.

class App_WithDoc : public App
{
public: 
	App_WithDoc(const string& appname, const string& title); //Don't forget to call init() too.
	virtual ~App_WithDoc();

	virtual void init(); //overriden to create document.
	 
protected:
  virtual void init_menus_file(); //overridden to add open/save/save as.
  virtual void init_toolbars(); //overridden to add open/save
	virtual void init_create_document(); //override this to new() the specific document type.

  virtual Document* get_document(); //static_cast<> or dynamic_cast<> this pointer to the correct type.

  //Signal handlers:

  //Menu items:
  virtual void on_menu_File_Open();
  virtual void on_menu_File_SaveAs(); //signal handler.
  virtual void offer_SaveAs(); //For direct use.
  virtual void on_menu_File_Save(Gtk::Widget* pWidget = 0); //signal handler.
  virtual void on_menu_File_Close();
  virtual void on_menu_File_Exit();

  virtual void on_menu_Edit_Copy();
  virtual void on_menu_Edit_Paste();
  virtual void on_menu_Edit_Clear();
  
  //Document:
  virtual void on_Document_modified(); //Update visual status.
  virtual void on_Document_load(); //override this to show document contents.

  virtual void offer_to_save_changes();
  virtual void cancel_close_or_exit(); //Stop the File|Close or the File|Exit.
  virtual void close_mark_or_destroy(); //destroy it or mark it for destruction.
  virtual void exit_close_next_instance(); //During a File|Exit tell the next instance to close too.
  virtual void exit_destroy_marked_instances(); //Destroy any marked instances, at the end of a File|Exit.
  
  virtual void update_window_title();
  virtual void after_successful_save(); //e.g. disable File|Save.
  

  //Document:
  Document* m_pDocument; //An instance of a derived type.
  bool m_bCloseAfterSave;
  bool m_bMarkedForDestroy; //Whether the instance should be destroyed soon.

  //Member widets:
  Bakery::FileSelection m_FileSelection_Open, m_FileSelection_Save;

  //Member widets:
  Gnome::AppBar m_Status;

  Gtk::Widget* m_pMenuItem_Save;
};

} //namespace

#endif
